/*
 * gmp3info.c
 *
 * GTK MP3info's main file. MP3info is an ID3 tag editor.
 *
 * Author:
 *       Ricardo Cerqueira	<rmc@rccn.net>
 *
 * This software is covered by the GNU public license, which should be in a file named
 * LICENSE acompanying this.
 *
 */

#include <gtk/gtk.h>
#include "mp3info.h"

GtkWidget	*id3win,*id3win_frame_global;
GtkWidget	*id3win_frame_title,*id3win_frame_artist,*id3win_frame_album;
GtkWidget	*id3win_frame_year,*id3win_frame_comment, *id3win_frame_albyear;
GtkWidget	*id3win_text_title,*id3win_text_artist,*id3win_text_album;
GtkWidget	*id3win_text_year,*id3win_text_comment;
GtkWidget	*id3win_frame_buttons,*id3win_ok_button,*id3win_cancel_button;
gchar		title[31]="",artist[31]="",album[31]="",year[4]="",comment[31]="",fbuf[4];
char fulltag[128];
FILE  *fp;
unsigned short sig;

char *gpad (char *string, int length) {
	while(strlen(string)<length) {
		strcat(string," ");
	}
	return string;
}

void unpad (char *string) {
	int x;
	for (x=(strlen(string)-1); x>=0; x--) {
		if (string[x]!=' ') {
			return;
		} else {
			string[x]='\0';
		}
	}
}
void exit_save( GtkWidget *widget, GtkWidget *data) {
	strcpy(fulltag,"TAG");
        strcat(fulltag,gpad(gtk_entry_get_text(GTK_ENTRY(id3win_text_title)),30));
        strcat(fulltag,gpad(gtk_entry_get_text(GTK_ENTRY(id3win_text_artist)),30));       
        strcat(fulltag,gpad(gtk_entry_get_text(GTK_ENTRY(id3win_text_album)),30));
        strcat(fulltag,gpad(gtk_entry_get_text(GTK_ENTRY(id3win_text_year)),4));
        strcat(fulltag,gpad(gtk_entry_get_text(GTK_ENTRY(id3win_text_comment)),30));
	strcat(fulltag,"");
        fwrite(fulltag,1,128,fp);
        fclose(fp);
	gtk_main_quit();
}

int main(int argc, char *argv[] ) {

   if(argc==2) { 
    if ( !( fp=fopen(argv[1],"rb+") ) ) {
            perror("Error opening MP3");
            return 0;
    }

    fread(&sig,sizeof(short),1,fp);
    sig &= 0xf0ff;                                                          
    if(sig != 0xf0ff) {  
            perror("File is not an MP3!");        
            return 0;
    }

    if ( fseek(fp, -128, SEEK_END )) { return 0; }
    fread(fbuf,1,3,fp); fbuf[3] = '\0';
    if (!strcmp("TAG",fbuf)) {
 	   fseek(fp, -125, SEEK_END); 
 	   fread(title,1,30,fp); unpad(title);
 	   fread(artist,1,30,fp); unpad(artist);
 	   fread(album,1,30,fp); unpad(album);
 	   fread(year,1,4,fp); unpad(year);
 	   fread(comment,1,30,fp); unpad(comment);
            fseek(fp, -128, SEEK_END);
    } else {
 	   fseek(fp, 0, SEEK_END); 
    }
  
	gtk_init (&argc, &argv);

	id3win=gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_window_set_title(GTK_WINDOW(id3win),"MP3info - An ID3 tag editor");
	gtk_window_set_policy (GTK_WINDOW(id3win), FALSE,FALSE,FALSE);
	gtk_widget_set_usize(id3win,450,300);
	gtk_container_border_width(GTK_CONTAINER(id3win),10);
	gtk_signal_connect(GTK_OBJECT(id3win), "delete_event", (GtkSignalFunc) gtk_exit, NULL);
	id3win_frame_global=gtk_vbox_new(FALSE,10);
	
        id3win_frame_title=gtk_frame_new("Title");
        gtk_container_border_width(GTK_CONTAINER(id3win_frame_title),5);
        id3win_text_title=gtk_entry_new_with_max_length(30);
	gtk_entry_set_text(GTK_ENTRY(id3win_text_title),title);
        gtk_container_add(GTK_CONTAINER(id3win_frame_title),id3win_text_title);
	gtk_box_pack_start(GTK_BOX(id3win_frame_global),id3win_frame_title,TRUE,TRUE,0);

        id3win_frame_artist=gtk_frame_new("Artist");
        gtk_container_border_width(GTK_CONTAINER(id3win_frame_artist),5);
        id3win_text_artist=gtk_entry_new_with_max_length(30);
	gtk_entry_set_text(GTK_ENTRY(id3win_text_artist),artist);
        gtk_container_add(GTK_CONTAINER(id3win_frame_artist),id3win_text_artist);
	gtk_box_pack_start(GTK_BOX(id3win_frame_global),id3win_frame_artist,TRUE,TRUE,0);

	id3win_frame_albyear=gtk_hbox_new(FALSE,30);
        id3win_frame_album=gtk_frame_new("Album");
        gtk_container_border_width(GTK_CONTAINER(id3win_frame_album),5);
        id3win_text_album=gtk_entry_new_with_max_length(30);
	gtk_entry_set_text(GTK_ENTRY(id3win_text_album),album);
        gtk_container_add(GTK_CONTAINER(id3win_frame_album),id3win_text_album);
	gtk_box_pack_start(GTK_BOX(id3win_frame_albyear),id3win_frame_album,TRUE,TRUE,0);

        id3win_frame_year=gtk_frame_new("Year");
	gtk_widget_set_usize(id3win_frame_year,2,0);
        gtk_container_border_width(GTK_CONTAINER(id3win_frame_year),5);
        id3win_text_year=gtk_entry_new_with_max_length(4);
	gtk_entry_set_text(GTK_ENTRY(id3win_text_year),year);
        gtk_container_add(GTK_CONTAINER(id3win_frame_year),id3win_text_year);
	gtk_box_pack_start(GTK_BOX(id3win_frame_albyear),id3win_frame_year,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(id3win_frame_global),id3win_frame_albyear,TRUE,TRUE,0);

        id3win_frame_comment=gtk_frame_new("Comment");
        gtk_container_border_width(GTK_CONTAINER(id3win_frame_comment),5);
        id3win_text_comment=gtk_entry_new_with_max_length(30);
	gtk_entry_set_text(GTK_ENTRY(id3win_text_comment),comment);
        gtk_container_add(GTK_CONTAINER(id3win_frame_comment),id3win_text_comment);
	gtk_box_pack_start(GTK_BOX(id3win_frame_global),id3win_frame_comment,TRUE,TRUE,0);

	id3win_frame_buttons=gtk_hbox_new(TRUE,30);
	id3win_ok_button=gtk_button_new_with_label("OK");
        gtk_box_pack_start(GTK_BOX(id3win_frame_buttons),id3win_ok_button,TRUE,TRUE,0);
	id3win_cancel_button=gtk_button_new_with_label("Cancel");
        gtk_box_pack_start(GTK_BOX(id3win_frame_buttons),id3win_cancel_button,TRUE,TRUE,0);
	gtk_widget_set_usize(id3win_frame_buttons,30,20);
	gtk_box_pack_start(GTK_BOX(id3win_frame_global),id3win_frame_buttons,TRUE,TRUE,0);

        gtk_container_add(GTK_CONTAINER(id3win),id3win_frame_global);

	gtk_signal_connect (GTK_OBJECT (id3win_ok_button), "clicked", GTK_SIGNAL_FUNC (exit_save), NULL);
	gtk_signal_connect (GTK_OBJECT (id3win_cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_exit), NULL);
	
	gtk_widget_show_all(id3win);
	gtk_main();

	return(0);
   } else {
	   g_print("You must supply an MP3 file as an argument!\n");
	   return(0);
   }

			
}

